"use strict";

var ObjectWithListeners = require('../ObjectWithListeners');
var Plugin = require('./Plugin');
var GuiUtils = require('../gui/leftPanel/GuiUtils');
var GuiConnector = require('../GuiConnector');

var Promise = require("bluebird");

// noinspection JSUnusedLocalSymbols
var logger = require('../logger');


function PluginManager(options) {
  ObjectWithListeners.call(this);
  var self = this;
  self.setProject(options.project);
  self.setMap(options.map);
  self.setElement(options.element);
  self.setConfiguration(options.configuration);
  self._plugins = [];
  self._pluginOnResizeHandlers = {};
}

PluginManager.prototype = Object.create(ObjectWithListeners.prototype);
PluginManager.prototype.constructor = ObjectWithListeners;

PluginManager.prototype.setProject = function (project) {
  this._project = project;
};
PluginManager.prototype.setMap = function (map) {
  this._map = map;
};
PluginManager.prototype.getMap = function () {
  return this._map;
};
PluginManager.prototype.setElement = function (element) {
  this._element = element;
};
PluginManager.prototype.getElement = function () {
  return this._element;
};

PluginManager.prototype.setConfiguration = function (configuration) {
  this._configuration = configuration;
};

PluginManager.prototype.getConfiguration = function () {
  return this._configuration;
};

PluginManager.prototype.getPlugins = function () {
  return this._plugins;
};

PluginManager.prototype.addPlugin = function (options) {
  var self = this;
  var tabData = self.createTabForPlugin();
  var plugin;
  if (options instanceof Plugin) {
    plugin = options;
    plugin.setOptions({
      element: tabData.content,
      configuration: self.getConfiguration(),
      map: self.getMap()
    });
  } else {
    plugin = new Plugin({
      url: options.url,
      element: tabData.content,
      configuration: self.getConfiguration(),
      map: self.getMap()
    });
    plugin.addListener("onUnload", function () {
      tabData.content.parentNode.removeChild(tabData.content);
      tabData.title.parentNode.parentNode.removeChild(tabData.title.parentNode);
    });
  }
  self._plugins.push(plugin);
  return plugin.load().then(function () {
    tabData.title.innerHTML = plugin.getName();
    const tab = $(tabData.content);
    tab.css('overflow', 'auto');
    var adjustHeight = function () {
      tab.css('height', 'calc( 100vh - ' + tab.offset().top + 'px )');
    };

    self._pluginOnResizeHandlers[plugin.getPluginId()] = adjustHeight;
    GuiConnector.addWindowResizeEvent(adjustHeight);
    adjustHeight();
    return plugin;
  });
};

PluginManager.prototype.createTabForPlugin = function () {
  var self = this;
  var tabData = self._tabData;
  var guiUtils = new GuiUtils(self.getConfiguration());
  if (tabData === undefined) {
    self.getElement().style.width = "300px";
    self.getElement().style.height = "100%";
    self._tabData = guiUtils.createTabDiv({element: self.getElement(), id: "plugin_tab"});
    tabData = self._tabData;
    $(tabData.content).css('position', 'relative');
  }
  return guiUtils.createTab({
    title: "",
    content: "",
    tabData: tabData
  });
};
PluginManager.prototype.removePlugin = function (plugin) {
  var self = this;
  var found = false;
  for (var i = 0; i < self._plugins.length; i++) {
    if (plugin === self._plugins[i]) {
      self._plugins.splice(i, 1);
      found = true;
    }
  }
  if (!found) {
    return Promise.reject(new Error("Plugin not registered"));
  }
  GuiConnector.removeWindowResizeEvent(self._pluginOnResizeHandlers[plugin.getPluginId()]);
  return plugin.unload();
};

PluginManager.prototype.destroy = function () {
  var self = this;
  var promises = [];
  var plugins = self._plugins;
  for (var i = plugins.length - 1; i >= 0; i--) {
    promises.push(self.removePlugin(plugins[i]));
  }
  return Promise.all(promises);
};


module.exports = PluginManager;
